#include "include/controller.h"

#include <pico/stdio.h>
#include <pico/cyw43_arch.h>
#include <pico/mutex.h>
#include "include/wifi_operator.h"
#include "include/udp_server.h"

auto_init_mutex(wifi_mutex);

int controller_init(void)
{
    stdio_init_all();

    sleep_ms(1000);

    inputs_init();

    if(cyw43_arch_init())
    {
        return -1;
    }

    mutex_enter_blocking(&wifi_mutex);
    cyw43_arch_gpio_put(CYW43_WL_GPIO_LED_PIN, 1);
    mutex_exit(&wifi_mutex);

    wifi_operator_init_ap_mode();
    udp_server_init();

    controller.is_running = true;
}

static inline void update_time(void)
{
    static bool led_state = false;
    static double last_time = 0.0;
    double start_time = (double)clock() * 1000.0 / (double)CLOCKS_PER_SEC;
    controller.delta_time_ms = start_time - last_time;
    last_time = start_time;

    static double elapsed_time = 0.0;
    elapsed_time += controller.delta_time_ms;

    if(elapsed_time >= 1000)
    {
        elapsed_time = 0;

        mutex_enter_blocking(&wifi_mutex);
        cyw43_arch_gpio_put(CYW43_WL_GPIO_LED_PIN, led_state);
        mutex_exit(&wifi_mutex);

        led_state = !led_state;
    }
}

void controller_handle_inputs_outputs(void)
{
    update_time();

    get_inputs();

    mutex_enter_blocking(&wifi_mutex);
    udp_server_send();
    mutex_exit(&wifi_mutex);

    sleep_us(500);

    tight_loop_contents();
}

void controller_deinit(void)
{
    udp_server_exit();
}
