#include "udp_server.h"

#include <stdio.h>
#include <pico/cyw43_arch.h>
#include <lwip/def.h>
#include "include/controller.h"

udp_server_t udp_server;

void udp_server_init()
{
    udp_server.pcb = udp_new();
    if(!udp_server.pcb)
    {
        puts("Error creating UDP server");
        return;
    }

    err_t err = udp_bind(udp_server.pcb, IP_ADDR_ANY, UDP_SERVER_PORT);
    if(err != ERR_OK)
    {
        printf("Error binding UDP server: %d\n", err);
        return;
    }
}

void udp_server_exit(void)
{
    if(udp_server.pcb)
    {
        udp_remove(udp_server.pcb);
        udp_server.pcb = NULL;
    }
}

static inline void udp_server_send_data(const uint8_t *data, size_t length)
{
    ip4_addr_t remote_addr;
    ip4addr_aton(UDP_CLIENT_IP, &remote_addr);

    // Nombre maximum de tentatives
    const int MAX_RETRIES = 3;
    uint tries = 0;

    while(tries < MAX_RETRIES)
    {
        struct pbuf *p = pbuf_alloc(PBUF_TRANSPORT, length, PBUF_RAM);
        if(p == NULL)
        {
            printf("Try %d - Error allocating pbuf\n", ++tries);
            sleep_ms(100);  // Wait before retry
            continue;
        }

        memcpy(p->payload, (char *)data, length);
        err_t err = udp_sendto(udp_server.pcb, p, &remote_addr, UDP_CLIENT_PORT);
        pbuf_free(p);

        if (err == ERR_OK)
        {
            return;  // Succès
        }
        else
        {
            const char* error_msg;

            switch(err)
            {
              case ERR_MEM:
                error_msg = "Insuficient memory";
                break;

              case ERR_BUF:
                error_msg = "Buffer full";
                break;

              default:
                error_msg = "Unknown error";
            }

            printf("Try %d - Error send UDP: %d (%s)\n", ++tries, err, error_msg);

            sleep_ms(100);  // Wait before retry
        }
    }

    printf("Error sending data after %d tries...\n", tries);
}

void udp_server_send(void)
{
    uint8_t data[UDP_SERVER_BUFFER_SIZE];

    for(uint i = 0; i < 16; i++) data[i] = 0;

    data[UDP_PAYLOAD_X_AXIS_SPEED_BYTE] = *(uint8_t *)&controller.inputs_buffer.x_axis_speed;
    data[UDP_PAYLOAD_Y_AXIS_SPEED_BYTE] = *(uint8_t *)&controller.inputs_buffer.x_axis_speed;

    udp_server_send_data(data, UDP_SERVER_BUFFER_SIZE);
}