#include "include/wifi_operator.h"

#include <pico/cyw43_arch.h>
#include <lwip/netif.h>
#include <lwip/ip4_addr.h>

void wifi_operator_init_ap_mode(void)
{
    // Configuration IP pour le mode AP
    ip4_addr_t ip, netmask, gateway;
    IP4_ADDR(&ip, 192, 168, 128, 1);
    IP4_ADDR(&netmask, 255, 255, 255, 0);
    IP4_ADDR(&gateway, 192, 168, 128, 1);

    puts("Configuration WiFi en cours...");

    // Désactiver le mode d'économie d'énergie
    cyw43_wifi_pm(&cyw43_state, CYW43_NO_POWERSAVE_MODE);

    // Activer le mode AP
    cyw43_arch_enable_ap_mode(WIFI_OPERATOR_SSID, WIFI_OPERATOR_PASSWORD, CYW43_AUTH_WPA2_AES_PSK);

    // Attendre que l'interface soit prête
    int retry = 0;
    while (netif_default == NULL && retry < 10)
    {
        sleep_ms(500);
        retry++;
    }

    if (netif_default == NULL)
    {
        puts("Erreur: Interface réseau non initialisée");
        return;
    }

    // Configuration du canal WiFi
    cyw43_wifi_ap_set_channel(&cyw43_state, WIFI_OPERATOR_CHANNEL);

    // Configuration de l'interface réseau
    netif_set_up(netif_default);
    netif_set_link_up(netif_default);
    netif_set_addr(netif_default, &ip, &netmask, &gateway);

    puts("Configuration IP terminée!");
    puts("Point d'accès activé!");
    printf("SSID:%s\n", WIFI_OPERATOR_SSID);
    printf("Password:%s\n", WIFI_OPERATOR_PASSWORD);
    printf("Channel:%d\n", WIFI_OPERATOR_CHANNEL);
}
